@@ -39,17 +39,21 @@ struct _GstRTPBaseDepayloadPrivate
   GstClockTime npt_stop;
   gdouble play_speed;
   gdouble play_scale;
+  guint clock_base;
 
   gboolean discont;
   GstClockTime pts;
   GstClockTime dts;
   GstClockTime duration;
 
+  guint32 last_seqnum;
+  guint32 last_rtptime;
   guint32 next_seqnum;
 
   gboolean negotiated;
 
   GstCaps *last_caps;
+  GstEvent *segment_event;
 };
 
 /* Filter signals and args */
@@ -62,6 +66,7 @@ enum
 enum
 {
   PROP_0,
+  PROP_STATS,
   PROP_LAST
 };
 
@@ -73,6 +78,8 @@ static void gst_rtp_base_depayload_get_property (GObject * object,
 
 static GstFlowReturn gst_rtp_base_depayload_chain (GstPad * pad,
     GstObject * parent, GstBuffer * in);
+static GstFlowReturn gst_rtp_base_depayload_chain_list (GstPad * pad,
+    GstObject * parent, GstBufferList * list);
 static gboolean gst_rtp_base_depayload_handle_sink_event (GstPad * pad,
     GstObject * parent, GstEvent * event);
 
@@ -89,6 +96,8 @@ static void gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass *
     klass);
 static void gst_rtp_base_depayload_init (GstRTPBaseDepayload * rtpbasepayload,
     GstRTPBaseDepayloadClass * klass);
+static GstEvent *create_segment_event (GstRTPBaseDepayload * filter,
+    guint rtptime, GstClockTime position);
 
 GType
 gst_rtp_base_depayload_get_type (void)
@@ -131,6 +140,69 @@ gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass * klass)
   gobject_class->set_property = gst_rtp_base_depayload_set_property;
   gobject_class->get_property = gst_rtp_base_depayload_get_property;
 
+
+  /**
+   * GstRTPBaseDepayload:stats:
+   *
+   * Various depayloader statistics retrieved atomically (and are therefore
+   * synchroized with each other). This property return a GstStructure named
+   * application/x-rtp-depayload-stats containing the following fields relating to
+   * the last processed buffer and current state of the stream being depayloaded:
+   *
+   * <variablelist>
+   *   <varlistentry>
+   *     <term>clock-rate</term>
+   *     <listitem><para>#G_TYPE_UINT, clock-rate of the
+   *     stream</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>npt-start</term>
+   *     <listitem><para>#G_TYPE_UINT64, time of playback start
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>npt-stop</term>
+   *     <listitem><para>#G_TYPE_UINT64, time of playback stop
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>play-speed</term>
+   *     <listitem><para>#G_TYPE_DOUBLE, the playback speed
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>play-scale</term>
+   *     <listitem><para>#G_TYPE_DOUBLE, the playback scale
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>running-time-dts</term>
+   *     <listitem><para>#G_TYPE_UINT64, the last running-time of the
+   *      last DTS
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>running-time-pts</term>
+   *     <listitem><para>#G_TYPE_UINT64, the last running-time of the
+   *      last PTS
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>seqnum</term>
+   *     <listitem><para>#G_TYPE_UINT, the last seen seqnum
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>timestamp</term>
+   *     <listitem><para>#G_TYPE_UINT, the last seen RTP timestamp
+   *     </para></listitem>
+   *   </varlistentry>
+   * </variablelist>
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_STATS,
+      g_param_spec_boxed ("stats", "Statistics", "Various statistics",
+          GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   gstelement_class->change_state = gst_rtp_base_depayload_change_state;
 
   klass->packet_lost = gst_rtp_base_depayload_packet_lost;
@@ -157,6 +229,8 @@ gst_rtp_base_depayload_init (GstRTPBaseDepayload * filter,
   g_return_if_fail (pad_template != NULL);
   filter->sinkpad = gst_pad_new_from_template (pad_template, "sink");
   gst_pad_set_chain_function (filter->sinkpad, gst_rtp_base_depayload_chain);
+  gst_pad_set_chain_list_function (filter->sinkpad,
+      gst_rtp_base_depayload_chain_list);
   gst_pad_set_event_function (filter->sinkpad,
       gst_rtp_base_depayload_handle_sink_event);
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
@@ -168,6 +242,15 @@ gst_rtp_base_depayload_init (GstRTPBaseDepayload * filter,
   gst_pad_use_fixed_caps (filter->srcpad);
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
+  priv->npt_start = 0;
+  priv->npt_stop = -1;
+  priv->play_speed = 1.0;
+  priv->play_scale = 1.0;
+  priv->clock_base = -1;
+  priv->dts = -1;
+  priv->pts = -1;
+  priv->duration = -1;
+
   gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
 }
 
@@ -232,6 +315,12 @@ gst_rtp_base_depayload_setcaps (GstRTPBaseDepayload * filter, GstCaps * caps)
   else
     priv->play_scale = 1.0;
 
+  value = gst_structure_get_value (caps_struct, "clock-base");
+  if (value && G_VALUE_HOLDS_UINT (value))
+    priv->clock_base = g_value_get_uint (value);
+  else
+    priv->clock_base = -1;
+
   if (bclass->set_caps) {
     res = bclass->set_caps (filter, caps);
     if (!res) {
@@ -257,11 +346,10 @@ caps_not_changed:
 }
 
 static GstFlowReturn
-gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
+gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
+    GstRTPBaseDepayloadClass * bclass, GstBuffer * in)
 {
-  GstRTPBaseDepayload *filter;
   GstRTPBaseDepayloadPrivate *priv;
-  GstRTPBaseDepayloadClass *bclass;
   GstFlowReturn ret = GST_FLOW_OK;
   GstBuffer *out_buf;
   GstClockTime pts, dts;
@@ -271,7 +359,6 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   gint gap;
   GstRTPBuffer rtp = { NULL };
 
-  filter = GST_RTP_BASE_DEPAYLOAD (parent);
   priv = filter->priv;
 
   /* we must have a setcaps first */
@@ -297,6 +384,9 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   rtptime = gst_rtp_buffer_get_timestamp (&rtp);
   gst_rtp_buffer_unmap (&rtp);
 
+  priv->last_seqnum = seqnum;
+  priv->last_rtptime = rtptime;
+
   discont = buf_discont;
 
   GST_LOG_OBJECT (filter, "discont %d, seqnum %u, rtptime %u, pts %"
@@ -345,7 +435,12 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
     }
   }
 
-  bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (filter);
+  /* prepare segment event if needed */
+  if (filter->need_newsegment) {
+    priv->segment_event = create_segment_event (filter, rtptime,
+        GST_BUFFER_PTS (in));
+    filter->need_newsegment = FALSE;
+  }
 
   if (G_UNLIKELY (bclass->process == NULL))
     goto no_process;
@@ -355,7 +450,6 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   if (out_buf) {
     ret = gst_rtp_base_depayload_push (filter, out_buf);
   }
-  gst_buffer_unref (in);
 
   return ret;
 
@@ -371,7 +465,6 @@ not_negotiated:
             "element before the depayloader and setting the 'caps' property "
             "on that. Also see http://cgit.freedesktop.org/gstreamer/"
             "gst-plugins-good/tree/gst/rtp/README"));
-    gst_buffer_unref (in);
     return GST_FLOW_NOT_NEGOTIATED;
   }
 invalid_buffer:
@@ -379,13 +472,11 @@ invalid_buffer:
     /* this is not fatal but should be filtered earlier */
     GST_ELEMENT_WARNING (filter, STREAM, DECODE, (NULL),
         ("Received invalid RTP payload, dropping"));
-    gst_buffer_unref (in);
     return GST_FLOW_OK;
   }
 dropping:
   {
     GST_WARNING_OBJECT (filter, "%d <= 100, dropping old packet", gap);
-    gst_buffer_unref (in);
     return GST_FLOW_OK;
   }
 no_process:
@@ -393,11 +484,68 @@ no_process:
     /* this is not fatal but should be filtered earlier */
     GST_ELEMENT_ERROR (filter, STREAM, NOT_IMPLEMENTED, (NULL),
         ("The subclass does not have a process method"));
-    gst_buffer_unref (in);
     return GST_FLOW_ERROR;
   }
 }
 
+static GstFlowReturn
+gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
+{
+  GstRTPBaseDepayloadClass *bclass;
+  GstRTPBaseDepayload *basedepay;
+  GstFlowReturn flow_ret;
+
+  basedepay = GST_RTP_BASE_DEPAYLOAD_CAST (parent);
+
+  bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (basedepay);
+
+  flow_ret = gst_rtp_base_depayload_handle_buffer (basedepay, bclass, in);
+
+  gst_buffer_unref (in);
+
+  return flow_ret;
+}
+
+static GstFlowReturn
+gst_rtp_base_depayload_chain_list (GstPad * pad, GstObject * parent,
+    GstBufferList * list)
+{
+  GstRTPBaseDepayloadClass *bclass;
+  GstRTPBaseDepayload *basedepay;
+  GstFlowReturn flow_ret;
+  GstBuffer *buffer;
+  guint i, len;
+
+  basedepay = GST_RTP_BASE_DEPAYLOAD_CAST (parent);
+
+  bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (basedepay);
+
+  flow_ret = GST_FLOW_OK;
+
+  /* chain each buffer in list individually */
+  len = gst_buffer_list_length (list);
+
+  if (len == 0)
+    goto done;
+
+  for (i = 0; i < len; i++) {
+    buffer = gst_buffer_list_get (list, i);
+
+    /* Should we fix up any missing timestamps for list buffers here
+     * (e.g. set to first or previous timestamp in list) or just assume
+     * the's a jitterbuffer that will have done that for us? */
+    flow_ret = gst_rtp_base_depayload_handle_buffer (basedepay, bclass, buffer);
+    if (flow_ret != GST_FLOW_OK)
+      break;
+  }
+
+done:
+
+  gst_buffer_list_unref (list);
+
+  return flow_ret;
+}
+
 static gboolean
 gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
     GstEvent * event)
@@ -410,6 +558,7 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
       gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
       filter->need_newsegment = TRUE;
       filter->priv->next_seqnum = -1;
+      gst_event_replace (&filter->priv->segment_event, NULL);
       break;
     case GST_EVENT_CAPS:
     {
@@ -439,7 +588,7 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
         /* we get this event from the jitterbuffer when it considers a packet as
          * being lost. We send it to our packet_lost vmethod. The default
          * implementation will make time progress by pushing out a GAP event.
-         * Subclasses can override and to one of the following:
+         * Subclasses can override and do one of the following:
          *  - Adjust timestamp/duration to something more accurate before
          *    calling the parent (default) packet_lost method.
          *  - do some more advanced error concealing on the already received
@@ -483,28 +632,59 @@ gst_rtp_base_depayload_handle_sink_event (GstPad * pad, GstObject * parent,
 }
 
 static GstEvent *
-create_segment_event (GstRTPBaseDepayload * filter, GstClockTime position)
+create_segment_event (GstRTPBaseDepayload * filter, guint rtptime,
+    GstClockTime position)
 {
   GstEvent *event;
-  GstClockTime stop;
+  GstClockTime start, stop, running_time;
   GstRTPBaseDepayloadPrivate *priv;
   GstSegment segment;
 
   priv = filter->priv;
 
+  /* determining the start of the segment */
+  start = 0;
+  if (priv->clock_base != -1 && position != -1) {
+    GstClockTime exttime, gap;
+
+    exttime = priv->clock_base;
+    gst_rtp_buffer_ext_timestamp (&exttime, rtptime);
+    gap = gst_util_uint64_scale_int (exttime - priv->clock_base,
+        filter->clock_rate, GST_SECOND);
+
+    /* account for lost packets */
+    if (position > gap) {
+      GST_DEBUG_OBJECT (filter,
+          "Found gap of %" GST_TIME_FORMAT ", adjusting start: %"
+          GST_TIME_FORMAT " = %" GST_TIME_FORMAT " - %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (gap), GST_TIME_ARGS (position - gap),
+          GST_TIME_ARGS (position), GST_TIME_ARGS (gap));
+      start = position - gap;
+    }
+  }
+
+  /* determining the stop of the segment */
+  stop = -1;
   if (priv->npt_stop != -1)
-    stop = priv->npt_stop - priv->npt_start;
-  else
-    stop = -1;
+    stop = start + (priv->npt_stop - priv->npt_start);
+
+  if (position == -1)
+    position = 0;
+
+  running_time = gst_segment_to_running_time (&filter->segment,
+      GST_FORMAT_TIME, start);
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   segment.rate = priv->play_speed;
   segment.applied_rate = priv->play_scale;
-  segment.start = 0;
+  segment.start = start;
   segment.stop = stop;
   segment.time = priv->npt_start;
   segment.position = position;
+  segment.base = running_time;
 
+  GST_DEBUG_OBJECT (filter, "Creating segment event %" GST_SEGMENT_FORMAT,
+      &segment);
   event = gst_event_new_segment (&segment);
 
   return event;
@@ -570,14 +750,9 @@ gst_rtp_base_depayload_prepare_push (GstRTPBaseDepayload * filter,
   }
 
   /* if this is the first buffer send a NEWSEGMENT */
-  if (G_UNLIKELY (filter->need_newsegment)) {
-    GstEvent *event;
-
-    event = create_segment_event (filter, 0);
-
-    gst_pad_push_event (filter->srcpad, event);
-
-    filter->need_newsegment = FALSE;
+  if (G_UNLIKELY (filter->priv->segment_event)) {
+    gst_pad_push_event (filter->srcpad, filter->priv->segment_event);
+    filter->priv->segment_event = NULL;
     GST_DEBUG_OBJECT (filter, "Pushed newsegment event on this first buffer");
   }
 
@@ -638,7 +813,7 @@ gst_rtp_base_depayload_push_list (GstRTPBaseDepayload * filter,
   return res;
 }
 
-/* convert the PacketLost event form a jitterbuffer to a GAP event.
+/* convert the PacketLost event from a jitterbuffer to a GAP event.
  * subclasses can override this.  */
 static gboolean
 gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
@@ -683,9 +858,11 @@ gst_rtp_base_depayload_change_state (GstElement * element,
       priv->npt_stop = -1;
       priv->play_speed = 1.0;
       priv->play_scale = 1.0;
+      priv->clock_base = -1;
       priv->next_seqnum = -1;
       priv->negotiated = FALSE;
       priv->discont = FALSE;
+      gst_event_replace (&filter->priv->segment_event, NULL);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -709,6 +886,29 @@ gst_rtp_base_depayload_change_state (GstElement * element,
   return ret;
 }
 
+static GstStructure *
+gst_rtp_base_depayload_create_stats (GstRTPBaseDepayload * depayload)
+{
+  GstRTPBaseDepayloadPrivate *priv;
+  GstStructure *s;
+
+  priv = depayload->priv;
+
+  s = gst_structure_new ("application/x-rtp-depayload-stats",
+      "clock_rate", G_TYPE_UINT, depayload->clock_rate,
+      "npt-start", G_TYPE_UINT64, priv->npt_start,
+      "npt-stop", G_TYPE_UINT64, priv->npt_stop,
+      "play-speed", G_TYPE_DOUBLE, priv->play_speed,
+      "play-scale", G_TYPE_DOUBLE, priv->play_scale,
+      "running-time-dts", G_TYPE_UINT64, priv->dts,
+      "running-time-pts", G_TYPE_UINT64, priv->pts,
+      "seqnum", G_TYPE_UINT, (guint) priv->last_seqnum,
+      "timestamp", G_TYPE_UINT, (guint) priv->last_rtptime, NULL);
+
+  return s;
+}
+
+
 static void
 gst_rtp_base_depayload_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -724,7 +924,15 @@ static void
 gst_rtp_base_depayload_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
+  GstRTPBaseDepayload *depayload;
+
+  depayload = GST_RTP_BASE_DEPAYLOAD (object);
+
   switch (prop_id) {
+    case PROP_STATS:
+      g_value_take_boxed (value,
+          gst_rtp_base_depayload_create_stats (depayload));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
